--Broker


--Broker Notification Services
USE AdventureWorks
GO
-- Create the queue to receive messages
CREATE QUEUE NotifyQueue
GO

--Create a service on the queue that references
--the event notifications contract.
CREATE SERVICE NotifyService ON QUEUE NotifyQueue
([http://schemas.microsoft.com/SQL/Notifications/PostEventNotification]);
GO

--Create a route on the service to define the address 
--to which Service Broker sends messages for the service.
CREATE ROUTE NotifyRoute WITH SERVICE_NAME = 'NotifyService', ADDRESS = 'LOCAL';

--Create a database-scoped event notification
CREATE EVENT NOTIFICATION Notify_ALTER_T1
ON DATABASE
FOR CREATE_TABLE
TO SERVICE 'NotifyService',
    '8140a771-3c4b-4479-8ac0-81008ab17984';

--Create a server-scoped event notification
CREATE EVENT NOTIFICATION Notify_ALTER_DB
ON SERVER
FOR ALTER_DATABASE
TO SERVICE 'NotifyService',
    '8140a771-3c4b-4479-8ac0-81008ab17984';
	
--Alter a database table
USE AdventureWorks2008
GO
ALTER TABLE Sales.CreditCard ADD IsDebitCard BINARY NULL
	
--Alter a database
ALTER DATABASE AdventureWorksDW
COLLATE SQL_Latin1_General_CP1_CI_AS 
GO

--Get information about ALTER TABLE notifications
SELECT * FROM sys.event_notifications
WHERE name = 'Notify_ALTER_T1'

--Get information about ALTER DATABASE notifications
SELECT * FROM sys.server_event_notifications
WHERE name = 'Notify_ALTER_DB'



--CTE

USE AdventureWorks;
GO
WITH ContactFullName (ContactID, FullName)
AS
(
SELECT ContactID, FirstName +
COALESCE(' ' + MiddleName + ' ', ' ') + LastName
FROM Person.Contact
)
SELECT * FROM ContactFullName;



CREATE VIEW Person.ContactFullName (ContactID, FullName)
AS
SELECT ContactID, FirstName +
COALESCE(' ' + MiddleName + ' ', ' ') + LastName
FROM Person.Contact;
GO
SELECT * FROM Person.ContactFullName;



SELECT *
FROM (SELECT ContactID, FirstName +
COALESCE(' ' + MiddleName + ' ', ' ') + LastName AS FullName
FROM Person.Contact) AS ContactFullName;



--CTE declaration
WITH EmpHierarchy (EmpID, MgrID, EmpLevel)
AS
(
-- CTE anchor query
SELECT EmployeeID, ManagerID, 1 AS EmpLevel
FROM HumanResources.Employee
WHERE ManagerID IS NULL
UNION ALL
-- CTE recursive query
SELECT e.EmployeeID, e.ManagerID, EmpLevel + 1
FROM HumanResources.Employee AS e
INNER JOIN EmpHierarchy AS d -- self-reference
ON e.ManagerID = d.EmpID
)
--referencing query
SELECT MgrID, COUNT(EmpID) AS EmpTotal, EmpLevel
FROM EmpHierarchy
WHERE MgrID IS NOT NULL
GROUP BY MgrID, EmpLevel
ORDER BY EmpLevel, MgrID;



--PIVOT TABLE
SELECT s.ProductID,
(SELECT SUM(d.LineTotal) FROM Sales.SalesOrderDetail AS d
JOIN Sales.SalesOrderHeader AS h
ON d.SalesOrderID = h.SalesOrderID
WHERE h.SalesPersonID = 268
AND d.ProductID = s.ProductID) AS [268],
(SELECT SUM(d.LineTotal) FROM Sales.SalesOrderDetail AS d
JOIN Sales.SalesOrderHeader AS h
ON d.SalesOrderID = h.SalesOrderID
WHERE h.SalesPersonID = 275
AND d.ProductID = s.ProductID) AS [275],
(SELECT SUM(d.LineTotal) FROM Sales.SalesOrderDetail AS d
JOIN Sales.SalesOrderHeader AS h
ON d.SalesOrderID = h.SalesOrderID
WHERE h.SalesPersonID = 276
AND d.ProductID = s.ProductID) AS [276]
FROM Sales.SalesOrderDetail AS s
GROUP BY s.ProductID;



SELECT d.ProductID,
SUM(CASE WHEN h.SalesPersonID = 268
THEN d.LineTotal ELSE NULL END) AS [268],
SUM(CASE WHEN h.SalesPersonID = 275
THEN d.LineTotal ELSE NULL END) AS [275],
SUM(CASE WHEN h.SalesPersonID = 276
THEN d.LineTotal ELSE NULL END) AS [276]
FROM Sales.SalesOrderDetail AS d
JOIN Sales.SalesOrderHeader AS h
ON d.SalesOrderID = h.SalesOrderID
GROUP BY d.productid;


SELECT * FROM
(SELECT h.SalesPersonID, d.ProductID, d.LineTotal
FROM Sales.SalesOrderDetail AS d
JOIN Sales.SalesOrderHeader AS h
ON d.SalesOrderID = h.SalesOrderID
WHERE salespersonid IS NOT NULL) As Sales
PIVOT (SUM(LineTotal)
FOR SalesPersonID IN ([268], [275], [276]))AS Pvt;

--unpivot
USE AdventureWorks;
GO
CREATE TABLE AccountBalances
(AcctID INT PRIMARY KEY, Jan MONEY, Feb MONEY, Mar MONEY, Apr MONEY,
May MONEY, Jun MONEY, Jul MONEY, Aug MONEY, Sep MONEY, Oct MONEY,
Nov MONEY, Dec MONEY);
INSERT INTO AccountBalances
VALUES (1234, 37.42, 22.22, 57.39, 18.22, 0, 97.67, .03,
47.00, 77.44, 45.45, 56.56, 67.67);
INSERT INTO AccountBalances
VALUES (1274, 7.42, 2.22, 7.39, 1.22, 7.00, 9.67, 7.03, 4.00,
7.44, 145.45, 156.56, 667.67);
INSERT INTO AccountBalances
VALUES (1288, 337.42, 322.22, 357.39, 318.22, 30.00,
397.67, 3.03, 347.00, 377.44, 345.45, 356.56, 367.67);
INSERT INTO AccountBalances
VALUES (1347, 437.42, 422.22, 457.39, 418.22, 440.00,
497.67, 4.03, 447.00, 477.44, 445.45, 456.56, 467.67);


SELECT AcctID, Jan AS SalesAmount, 'Jan' AS SalesMonth FROM
AccountBalances
UNION ALL
SELECT AcctID, Feb AS SalesAmount, 'Feb' AS SalesMonth FROM
AccountBalances
UNION ALL
SELECT AcctID, Mar AS SalesAmount, 'Mar' AS SalesMonth FROM
AccountBalances
UNION ALL
SELECT AcctID, Apr AS SalesAmount, 'Apr' AS SalesMonth FROM
AccountBalances
UNION ALL
SELECT AcctID, May AS SalesAmount, 'May' AS SalesMonth FROM
AccountBalances
ORDER BY AcctID;



SELECT * FROM
(SELECT AcctID, Jan, Feb, Mar, Apr, May
FROM AccountBalances) AS AcctBal
UNPIVOT (SalesAmount FOR SalesMonth
IN ([Jan], [Feb], [Mar], [Apr], [May])) AS Unpvt;


SELECT * FROM
(SELECT h.SalesPersonID, d.ProductID, d.LineTotal
FROM Sales.SalesOrderDetail AS d
JOIN Sales.SalesOrderHeader AS h
ON d.SalesOrderID = h.SalesOrderID
WHERE SalesPersonID IS NOT NULL) As Sales
PIVOT (SUM(LineTotal)
FOR SalesPersonID IN ([268], [275], [276])) AS Pvt;



--RANKING
SELECT SalesOrderID, SubTotal,
ROW_NUMBER() OVER(ORDER BY SubTotal DESC) AS RowNumber,
RANK() OVER(ORDER BY SubTotal DESC) AS Rank,
DENSE_RANK() OVER(ORDER BY SubTotal DESC) AS DenseRank,
NTILE(3) OVER(ORDER BY SubTotal DESC) AS NTile
FROM Sales.SalesOrderHeader
WHERE SalesOrderID IN (51885, 51886, 52031, 56410,
56411, 72585, 65101, 65814, 73976)
ORDER BY SubTotal DESC;


SELECT CustomerID, SalesOrderID, SubTotal,
DENSE_RANK() OVER(PARTITION BY CustomerID
ORDER BY SubTotal DESC) AS DenseRank
FROM Sales.SalesOrderHeader
WHERE CustomerID IN (427, 428, 429)
ORDER BY CustomerID, SubTotal DESC;

--paginacja


WITH PagedOrders
AS
(
SELECT SalesOrderID, CustomerID, SubTotal,
ROW_NUMBER() OVER(ORDER BY SalesOrderID) AS RowNumber
FROM Sales.SalesOrderHeader
)
SELECT SalesOrderID, CustomerID, SubTotal
FROM PagedOrders
WHERE RowNumber BETWEEN 11 AND 21;


CREATE TABLE [Sales].[SalesOrderHeader2](
	[SalesOrderID] [int] ,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderDate] [datetime] NOT NULL,
	[DueDate] [datetime] NOT NULL,
	[ShipDate] [datetime] NULL,
	[Status] [tinyint] NOT NULL,
	[OnlineOrderFlag] [dbo].[Flag] NOT NULL,
	[SalesOrderNumber]  VARCHAR(100),
	[PurchaseOrderNumber] [dbo].[OrderNumber] NULL,
	[AccountNumber] [dbo].[AccountNumber] NULL,
	[CustomerID] [int] NOT NULL,
	[SalesPersonID] [int] NULL,
	[TerritoryID] [int] NULL,
	[BillToAddressID] [int] NOT NULL,
	[ShipToAddressID] [int] NOT NULL,
	[ShipMethodID] [int] NOT NULL,
	[CreditCardID] [int] NULL,
	[CreditCardApprovalCode] [varchar](15) NULL,
	[CurrencyRateID] [int] NULL,
	[SubTotal] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[TotalDue]  money,
	[Comment] [nvarchar](128) NULL,
	[rowguid] [uniqueidentifier] ROWGUIDCOL  NOT NULL,
	[ModifiedDate] [datetime] NOT NULL)


INSERT INTO Sales.SalesOrderHeader2
SELECT * FROM Sales.SalesOrderHeader
WHERE SalesOrderID < 50000;

INSERT INTO Sales.SalesOrderHeader2
SELECT * FROM Sales.SalesOrderHeader
WHERE SalesOrderID < 44000;
INSERT INTO Sales.SalesOrderHeader2
SELECT * FROM Sales.SalesOrderHeader
WHERE SalesOrderID < 43800;

WITH OrderDuplicates
AS
(
SELECT SalesOrderID,
ROW_NUMBER() OVER(ORDER BY SalesOrderID) AS RowNumber,
RANK() OVER(ORDER BY SalesOrderID) AS Rank,
ROW_NUMBER() OVER(ORDER BY SalesOrderID) -
RANK() OVER(ORDER BY SalesOrderID) AS IsDuplicate
FROM Sales.SalesOrderHeader2
)
SELECT * FROM OrderDuplicates WHERE IsDuplicate > 0;



--XQUERY

SELECT Name, Demographics.value
('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/
StoreSurvey";
(/sv:StoreSurvey/sv:AnnualSales)[1] div
(12*(/sv:StoreSurvey/sv:NumberEmployees)[1])',
'numeric(11,2)') AS MonthlyEmpSalesRate
FROM Sales.Store
WHERE CustomerID = 1;


SELECT JobCandidateID, Resume.query
('declare namespace rs=
"http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/Resume";
<html><body><table>
{
for $e in /rs:Resume/rs:Employment
order by $e/rs:Emp.JobTitle
return
<tr>
<td>{fn:string($e/rs:Emp.JobTitle)}</td>
</tr>
}
</table></body></html>') AS EmpHistory
FROM HumanResources.JobCandidate
WHERE JobCandidateID = 1;



SELECT Name, Demographics.query
('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/
StoreSurvey";
if (fn:number((/sv:StoreSurvey/sv:AnnualRevenue)[1]) > 250000)
then "Sales greater than $250,000"
else "Annual sales less than or equal to $250,000" ') AS TotalSales
FROM Sales.Store
WHERE CustomerID = 1;


SELECT ProductModelID, Instructions.value
(' declare namespace
man="http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/
ProductModelManuInstructions";
some $m in //man:root/man:Location
satisfies $m/@MachineHours',
'nvarchar(10)') as Result
FROM Production.ProductModel
where Instructions IS NOT NULL;



DECLARE @x XML;
SET @x = '';
SELECT @x.query('"Book" instance of xs:integer');

DECLARE @x XML;
SET @x = '';
SELECT @x.query('123 cast as xs:string?');

--parametry

SELECT Name, Demographics.query
('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/
StoreSurvey";
<Store
StoreName="{sql:column("Name")}"
AnnualSales="{/sv:StoreSurvey/sv:AnnualSales}">
</Store>') AS StoreSales
FROM Sales.Store
WHERE CustomerID = 1;


DECLARE @goal MONEY
SET @goal = 400000
SELECT Name, Demographics.query
('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/
StoreSurvey";
<Store
StoreName="{sql:column("Name")}"
AnnualSales="{/sv:StoreSurvey/sv:AnnualSales}"
SalesGoal="{sql:variable("@goal")}">
</Store>') AS StoreSales
FROM Sales.Store
WHERE CustomerID = 1;


CREATE FUNCTION Sales.StoreRevenue (@p1 XML)
RETURNS NUMERIC(18,2) AS
BEGIN
DECLARE @Result NUMERIC(18,2)
SET @Result=@p1.value
('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/
adventure-works/StoreSurvey";
(/sv:StoreSurvey/sv:AnnualRevenue)[1]',
'numeric(18,2)')
RETURN @Result
END;

ALTER TABLE Sales.Store
ADD Revenue AS Sales.StoreRevenue(Demographics);


SELECT CustomerID, Name
FROM Sales.Store
WHERE Demographics.exist('declare namespace
sv="http://schemas.microsoft.com/sqlserver/2004/07/
adventure-works/StoreSurvey";
/sv:StoreSurvey/sv:Specialty[.="Road"]') = 1
ORDER BY CustomerID;

SELECT Name, Demographics.value
('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/
adventure-works/StoreSurvey";
(/sv:StoreSurvey/sv:AnnualSales)[1] div
(12*(/sv:StoreSurvey/sv:NumberEmployees)[1])',
'numeric(11,2)') AS MonthlyEmpSalesRate,
Demographics.value
('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/
adventure-works/StoreSurvey";
(/sv:StoreSurvey/sv:AnnualRevenue)[1] div
(12*(/sv:StoreSurvey/sv:NumberEmployees)[1])',
'numeric(11,2)') AS MonthlyAnnualRevenue
FROM Sales.Store
WHERE CustomerID = 1;

--3 rownowazne


SELECT CustomerID, Name
FROM Sales.Store
WHERE CAST(Demographics.query('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/
StoreSurvey";
data(/sv:StoreSurvey/sv:AnnualRevenue)') AS VARCHAR(24)
) = '80000';


SELECT CustomerID, Name
FROM Sales.Store
WHERE Demographics.value('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/
StoreSurvey";
data((/sv:StoreSurvey/sv:AnnualRevenue)[1])', 'int') = 80000;


SELECT CustomerID, Name
FROM Sales.Store
WHERE Demographics.exist('declare namespace sv=
"http://schemas.microsoft.com/sqlserver/2004/07/adventure-works/
StoreSurvey";
/sv:StoreSurvey/sv:AnnualRevenue[.=80000]') = 1;


--generowanie xml

SELECT header.SalesOrderID, header.AccountNumber,
detail.ProductID, detail.OrderQty, detail.UnitPrice
FROM Sales.SalesOrderHeader header
JOIN Sales.SalesOrderDetail detail
ON header.SalesOrderID = detail.SalesOrderID
WHERE header.SalesOrderID < 43662
FOR XML RAW('Order'), ELEMENTS, ROOT('SalesOrders');

SELECT header.SalesOrderID, header.AccountNumber,
detail.ProductID, detail.OrderQty, detail.UnitPrice
FROM Sales.SalesOrderHeader header
JOIN Sales.SalesOrderDetail detail
ON header.SalesOrderID = detail.SalesOrderID
WHERE header.SalesOrderID < 43662
FOR XML AUTO, ELEMENTS, ROOT('SalesOrders');


SELECT
1 AS Tag,
NULL AS Parent,
h.SalesOrderID AS [Order!1!OrderID!ID],
h.AccountNumber AS [Order!1!AccountNumber!element],
NULL AS [OrderDetail!2!ProductID!ID],
NULL AS [OrderDetail!2!Quantity!Element],
NULL AS [OrderDetail!2!Price!Element]
FROM Sales.SalesOrderHeader h
WHERE h.SalesOrderID < 43662
UNION ALL
SELECT
2,
1,
d.SalesOrderID,
NULL,
d.ProductID,
d.OrderQty,
d.UnitPrice
FROM Sales.SalesOrderDetail d
WHERE d.SalesOrderID < 43662
ORDER BY 3, 5
FOR XML EXPLICIT;


SELECT h.SalesOrderID AS "@OrderID",
h.AccountNumber AS "AccountNumber",
d.ProductID AS "OrderDetail/@ProductID",
d.OrderQty AS "OrderDetail/@Quantity",
d.UnitPrice AS "OrderDetail/@Price"
FROM Sales.SalesOrderHeader h
JOIN Sales.SalesOrderDetail d
ON h.SalesOrderID = d.SalesOrderID
WHERE h.SalesOrderID < 43662
FOR XML PATH('Order'), ROOT('SalesOrders');


--openxml
DECLARE @docHandle INT, @xmlDoc XML;
SET @xmlDoc =
'<Orders>
<Order OrderID="4568">
<Customer CustomerID="1234" CompanyName="Company1"/>
<OrderDate>2005-01-25T00:00:00</OrderDate>
<OrderDetails>
<Line ProductID="812">
<Quantity>12</Quantity>
<Price>12.35</Price>
</Line>
<Line ProductID="815">
<Quantity>3</Quantity>
<Price>28.15</Price>
</Line>
</OrderDetails>
</Order>
<Order OrderID="4569">
<Customer CustomerID="1235" CompanyName="Company2" />
<OrderDate>2005-01-26T00:00:00</OrderDate>
<OrderDetails>
<Line ProductID="816">
<Quantity>6</Quantity>
<Price>5.45</Price>
</Line>
<Line ProductID="815">
<Quantity>8</Quantity>
<Price>19.99</Price>
</Line>
</OrderDetails>
</Order>
</Orders>';
EXEC sp_xml_preparedocument @docHandle OUTPUT, @xmlDoc;
SELECT * FROM
OPENXML(@docHandle, N'//Orders/Order')
WITH (OrderID INT '@OrderID',
OrderDate SMALLDATETIME 'OrderDate');



SELECT
Doc.Ord.value('@OrderID', 'INT') AS OrderID,
Doc.Ord.value('OrderDate[1]','SMALLDATETIME') AS OrderDate
FROM @xmlDoc.nodes('/Orders/Order') AS Doc(Ord);


